<?php

namespace OtomaticAi\Controllers;

use Exception;
use OtomaticAi\Api\OtomaticAi\Client;
use OtomaticAi\Models\Preset;
use OtomaticAi\Vendors\Carbon\Carbon;
use OtomaticAi\Vendors\Illuminate\Support\Arr;
use OtomaticAi\Vendors\Illuminate\Support\Str;

class TextBlocksController extends Controller
{
    public function index()
    {
        $this->verifyNonce();

        $this->response([
            "local" => [],
            "otomatic_ai" => $this->getOtomaticAITextBlocks(),
        ]);
    }

    public function generatePrompt()
    {
        $this->verifyNonce();

        $this->validate([
            "description" => ["required", "string"],
        ]);

        try {
            // get the openai preset
            $preset = Preset::findFromAPI("generate_custom_prompt");

            // run the preset
            $response = $preset->process([
                "content" => $this->input('description'),
            ]);

            // get the response content
            $content = Arr::get($response, 'choices.0.message.content');
            $content = Str::clean($content);

            if (!empty($content)) {
                $this->response([
                    "prompt" => $content,
                ]);
            }
        } catch (Exception $e) {
            $this->response(["message" => "An error occurred", "error" => $e->getMessage()], 503);
        }

        $this->response([
            "prompt" => "",
        ]);
    }

    private function getOtomaticAITextBlocks()
    {
        $optionName = "otomatic_ai_text_blocks";

        $output = [];

        if (function_exists('get_option')) {
            $cache = get_option($optionName, null);

            // fetch or refresh cache
            if (
                empty($cache) ||
                Carbon::createFromFormat(
                    "Y-m-d H:i:s",
                    Arr::get(
                        $cache,
                        "cached_at",
                        Carbon::now()->subMonth()
                    )
                )
                ->lt(
                    Carbon::now()->subMinutes(5)
                )
            ) {
                try {
                    $api = new Client;
                    $output = $api->textBlocks();

                    $cache = [
                        "data" => $output,
                        "cached_at" => Carbon::now()->format("Y-m-d H:i:s"),
                    ];

                    if (function_exists('update_option')) {
                        update_option($optionName, $cache, false);
                    }
                } catch (Exception $e) {
                }
            } else {
                $output = Arr::get($cache, "data", $output);
            }
        }

        return $output;
    }
}
